# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-15 06:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('async_notifications', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailnotification',
            options={'verbose_name': 'Email notification', 'verbose_name_plural': 'Email notifications'},
        ),
        migrations.AlterModelOptions(
            name='emailtemplate',
            options={'verbose_name': 'Email template', 'verbose_name_plural': 'Email templates'},
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='code',
            field=models.CharField(max_length=50, verbose_name='Code'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class NotificationsConfig(AppConfig):
    name = 'async_notifications'
    verbose_name = _("Email Notification")

    def ready(self):
        import async_notifications.signals



# encoding: utf-8

'''
Free as freedom will be 25/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from ajax_select.fields import (AutoCompleteSelectMultipleField,
                                AutoCompleteSelectMultipleWidget)


class EmailLookupWidget(AutoCompleteSelectMultipleWidget):

    def value_from_datadict(self, data, files, name):
        # eg. 'members': ['|229|4688|190|']
        # backward compatibility ['1,3,4']
        ids = data.get(name, '')
        if "," in ids:
            return [val for val in ids.split(',') if val]

        return super(EmailLookupWidget, self).value_from_datadict(
            data, files, name)


class EmailLookup(AutoCompleteSelectMultipleField):
    pass



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from .models import TemplateContext, EmailTemplate
from django.template.loader import render_to_string
import json

codes = []

class DummyContextObject(object):
    def __init__(self, name):
        self.parent = name

    def __getattr__(self, name):
        return DummyContextObject("%s.%s" % (self.parent, name))

    def __str__(self, *args, **kwargs):
        return "{{ %s }}" % (self.parent)

    def __repr__(self, *args, **kwargs):
        return "{{ %s }}" % (self.parent)


def update_template_context(code, view_name,  context, message=" ", as_template=False):
    """ context :
        [
        (name, help_text),
        ]

    """
    if as_template:
        new_context = {}
        for c in context:
            if not isinstance(c, str):
                c=c[0]
            new_context[c] = DummyContextObject(c)
        message = render_to_string(message, context=new_context)
    ncode = "%s.%s" % (code, view_name)
    try:  # fail ok if no migrations first python manage.py migrate
        if ncode not in codes:
            temp_context, created = TemplateContext.objects.get_or_create(
                code=code)
            try:
                EmailTemplate.objects.get(code=code)
            except:
                EmailTemplate.objects.create(
                    code=code, subject=view_name, message=message)

            if temp_context.context_dic:
                context_dic = json.loads(temp_context.context_dic)
            else:
                context_dic = {}

            if view_name not in context_dic:
                context_dic[view_name] = context
                temp_context.context_dic = json.dumps(context_dic)
                temp_context.save()
                codes.append("%s.%s" % (code, view_name))
    except:
        pass



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals
import base64


def get_app_name_models(model):
    return "{0}.{1}".format(model._meta.app_label, model._meta.object_name).lower()


def get_base64url(url):
    return base64.b64encode(str.encode(url))


def get_str_base64url(url):
    return base64.b64decode(str.encode(url))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('async_notifications', '0002_auto_20160515_0018'),
    ]

    operations = [
        migrations.CreateModel(
            name='TemplateContext',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('code', models.CharField(verbose_name='Code', max_length=50)),
                ('context_dic', models.TextField(verbose_name='Context dictionary')),
            ],
            options={
                'verbose_name': 'Context of template',
                'verbose_name_plural': 'Context of template',
            },
        ),
    ]



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from classytags.core import Options
from classytags.helpers import Tag
from classytags.arguments import Argument
from django import template
from django.template.loader import render_to_string

from async_notifications.djcms_async_notifications.forms import NotificationForm
from async_notifications.djcms_async_notifications.utils import (
    get_app_name_models, get_base64url)


register = template.Library()


class Render_email_notification(Tag):
    name = 'render_email_notification'
    options = Options(
        Argument('page', required=True),
    )

    def render_tag(self, context, page):
        if 'request' not in context:
            return ""
        else:
            request = context['request']
            user = request.user

        if not user.is_superuser:
            return ""

        name = get_app_name_models(page)

        uri = request.build_absolute_uri()
        uri = get_base64url(uri)
        return render_to_string(
            'djcms_async_notifications/page_email.html',
            {'request': request,
             'uri': uri,
             'name': name,
             'pk': page.pk},
            request=request
        )

register.tag(Render_email_notification)



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from cms.utils.urlutils import admin_reverse

from django.utils.translation import ugettext_lazy as _
from async_notifications.models import EmailNotification


@toolbar_pool.register
class EmailToolbar(CMSToolbar):
    watch_models = [EmailNotification]
    supported_apps = ('djcms_async_notifications', )

    def populate(self):
        user = getattr(self.request, 'user', None)
        try:
            view_name = self.request.resolver_match.view_name
        except AttributeError:
            view_name = None

        if user and view_name:
            menu = self.toolbar.get_or_create_menu(
                'async-notification-app', _("Email"))
            change_email_perm = user.has_perm(
                'async_notifications.change_emailnotification')
            add_email_perm = user.has_perm(
                'async_notifications.add_emailnotification')

            change_template_perm = user.has_perm(
                'async_notifications.change_emailtemplate')
            add_template_perm = user.has_perm(
                'async_notifications.add_emailtemplate')

            if add_email_perm:
                url = admin_reverse(
                    'async_notifications_emailnotification_add')
                menu.add_sideframe_item(_('Write email'), url=url)

            if change_email_perm:
                url = admin_reverse(
                    'async_notifications_emailnotification_changelist')
                menu.add_sideframe_item(_('Email list'), url=url)

            menu.add_break()

            if change_template_perm:
                url = admin_reverse(
                    'async_notifications_emailtemplate_changelist')
                menu.add_sideframe_item(_('Email template list'), url=url)

            if add_template_perm:
                url = admin_reverse(
                    'async_notifications_emailtemplate_add')
                menu.add_sideframe_item(_('Write template email'), url=url)


# encoding: utf-8


'''
Created on 18/11/2016

@author: luisza
'''
from __future__ import unicode_literals

from async_notifications.register import update_template_context
from async_notifications.settings import CONTACT_PLUS_EMAIL
from async_notifications.utils import send_email_from_template


context = [
    ('from_email', 'From email for header'),
    ('context',
        """ {data: form data, ordered_data: form data in order, instance: form plugin}""" ),
    ('subject', "Email subject"),
    ('instance', "form plugin instance"),
    ('headers', "Http headers (don't use)"),
    ('to',  "to email list")
]


def send_email(**kwargs):
    context_name = "contact_plus"
    update_template_context("fromsite_" + context_name,
                            'New contact form submited',
                            context
                            )

    update_template_context("sitetouser_" + context_name,
                            'Answer for contact form submited',
                            context
                            )
    send_email_from_template("fromsite_" + context_name,
                             kwargs['to'],
                             context=kwargs,
                             enqueued=True)

    send_email_from_template("sitetouser_" + context_name,
                             kwargs['context']['data'][CONTACT_PLUS_EMAIL],
                             context=kwargs,
                             enqueued=True)



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from django.conf import settings
import importlib


NOTIFICATION_GROUP_MODEL = getattr(
    settings, 'ASYNC_NOTIFICATION_GROUP', 'auth.Group')

GROUP_LOOKUP_FIELDS = getattr(
    settings, 'ASYNC_NOTIFICATION_GROUP_LOOKUP_FIELDS',
    {'order_by': 'name',
     'email': None,
     'display': 'name',
     'group_lookup': 'name',
     'filter': ['name__icontains']})


AUTH_USER = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
NOTIFICATION_USER_MODEL = getattr(
    settings, 'ASYNC_NOTIFICATION_USER', AUTH_USER)

USER_LOOKUP_FIELDS = getattr(
    settings, 'ASYNC_NOTIFICATION_USER_LOOKUP_FIELDS',
    {'order_by': 'username',
     'display': 'username',
     'filter': ['username__icontains',
                'first_name__icontains',
                'last_name__icontains'],
     'group_lookup': 'groups__name'})


MAX_PER_MAIL = getattr(settings, 'ASYNC_NOTIFICATION_MAX_PER_MAIL', 40)

TEXT_AREA_WIDGET_TXT = getattr(settings, 'ASYNC_NOTIFICATION_TEXT_AREA_WIDGET',
                               'django.forms.widgets.Textarea')


module_name, class_name = TEXT_AREA_WIDGET_TXT.rsplit(".", 1)
TEXT_AREA_WIDGET = getattr(importlib.import_module(module_name), class_name)

CONTACT_PLUS_EMAIL = getattr(
    settings, 'ASYNC_NOTIFICATION_CONTACT_PLUS_EMAIL', 'email')

EXTRA_BCC = getattr(settings, 'ASYNC_BCC', None)
EXTRA_CC = getattr(settings, 'ASYNC_CC', None)


# encoding: utf-8
from __future__ import unicode_literals

from .settings import EXTRA_BCC, EXTRA_CC

'''
Created on 20/12/2015

@author: luisza
'''

from django.template import Context, Template
from .models import EmailTemplate, EmailNotification
from django.apps import apps
from django.template.exceptions import TemplateDoesNotExist
import six


def hexify(text):
    return "".join([str(hex(ord(x))).replace("0x", "") for x in text])


def unhexify(text):
    if "@" in text:
        return text
    return "".join([chr(int(text[x] + text[x + 1], base=16))
                    for x in range(0, len(text), 2)])

def _get_template(code):
    template = None
    if type(code) == six.text_type or type(code) == six.binary_type:
        code = [code]
    for _code in code:
        template = EmailTemplate.objects.filter(code=_code).first()
        if template:
            break
    if template is None:
        raise TemplateDoesNotExist()
    return template

def get_bcc_field(template, bcc):
    bcc = bcc or EXTRA_BCC
    if template.cc is not None:
        if bcc is None:
            bcc = template.cc
        else:
            bcc +=","+template.cc
    return bcc

def get_cc_field(template, cc):
    cc = cc or EXTRA_CC
    if template.cc is not None:
        if cc is None:
            cc = template.cc
        else:
            cc +=","+template.cc
    return cc

def send_email_from_template(code, recipient,
                             context={},
                             enqueued=True,
                             user=None,
                             upfile=None,
                             bcc=None,
                             cc=None):

    template = _get_template(code)
    if user is not None:
        context['user'] = user

    if type(recipient) is list or type(recipient) is tuple:
        recipient = ",".join(recipient)

    subject = Template(template.subject)
    message = Template(template.message)
    c = Context(context)
    bcc=get_bcc_field(template, bcc)
    cc=get_cc_field(template, cc)
    email = EmailNotification(subject=subject.render(c),
                              message=message.render(c),
                              recipient=recipient,
                              enqueued=enqueued,
                              bcc=bcc,
                              cc=cc
                              )
    if user is not None:
        email.user = user
    if upfile is not None:
        email.file = upfile

    email.save()


def extract_emails(text):
    if isinstance(text, str):
        mail_list = text.replace(
            "[", "").replace("]", "").replace("'", "").split(",")
    else:
        mail_list = text
        mail_list = [x.replace('[', '').replace(']', '').replace("'", '') for x in mail_list ]
    emails = [unhexify(x.strip()) for x in mail_list]
    return emails


def get_model(model_name):
    app_name, model = model_name.split(".")
    return apps.get_model(app_name, model)



from django.db import models

from django.conf import settings
# Create your models here.
from django.utils.translation import ugettext_lazy as _


class EmailNotification(models.Model):
    subject = models.CharField(max_length=500, verbose_name=_("Subject"))
    message = models.TextField(verbose_name=_("Message"))
    # coma separated
    recipient = models.TextField(verbose_name=_("Recipient list"))
    enqueued = models.BooleanField(default=True,
                                   verbose_name=_("Enqueued"))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True,
                             blank=True, verbose_name=_("User"), on_delete=models.CASCADE)
    sended = models.BooleanField(default=False, verbose_name=_("Sended"))
    file = models.FileField(upload_to='email/%Y/%M', null=True,
                            blank=True, verbose_name=_("File"))
    problems = models.BooleanField(default=False, verbose_name=_("Problems"))
    create_datetime = models.DateTimeField(auto_now_add=True,
                                           verbose_name=_("Create datetime"))

    bcc = models.TextField(verbose_name=_("Bcc Blind carbon copy list"), null=True, blank=True)
    cc = models.TextField(verbose_name=_("Cc Carbon Copy list"), null=True, blank=True)

    def __str__(self):
        return "(%s) %s" % (_("sended") if self.sended else _("Not sended"),
                            self.subject)

    class Meta:
        verbose_name = _("Email notification")
        verbose_name_plural = _("Email notifications")



class EmailTemplate(models.Model):
    code = models.CharField(max_length=50, verbose_name=_("Code"))
    subject = models.CharField(max_length=500, verbose_name=_("Subject"))
    message = models.TextField(verbose_name=_("Message"))
    bcc = models.TextField(verbose_name=_("Bcc Blind carbon copy list"), null=True, blank=True)
    cc = models.TextField(verbose_name=_("Cc Carbon Copy list"), null=True, blank=True)

    def __str__(self):
        return "(%s) %s" % (self.code, self.subject)

    class Meta:
        verbose_name = _("Email template")
        verbose_name_plural = _("Email templates")



class TemplateContext(models.Model):
    code = models.CharField(max_length=50, verbose_name=_("Code"))
    context_dic = models.TextField(verbose_name=_("Context dictionary"))

    def __str__(self):
        return self.code

    class Meta:
        verbose_name = _("Context of template")
        verbose_name_plural = _("Context of template")



# Generated by Django 3.0.3 on 2020-02-28 22:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('async_notifications', '0003_templatecontext'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='bcc',
            field=models.TextField(blank=True, null=True, verbose_name='Bcc Blind carbon copy list'),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='cc',
            field=models.TextField(blank=True, null=True, verbose_name='Cc Carbon Copy list'),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='bcc',
            field=models.TextField(blank=True, null=True, verbose_name='Bcc Blind carbon copy list'),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='cc',
            field=models.TextField(blank=True, null=True, verbose_name='Cc Carbon Copy list'),
        ),
    ]



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from async_notifications.widgets import EmailLookup
from async_notifications.models import EmailTemplate
from django import forms
from django.utils.translation import ugettext_lazy as _

Textarea = forms.widgets.Textarea


class NotificationForm(forms.Form):
    template = forms.ModelChoiceField(queryset=EmailTemplate.objects.all(),
                                      label=_('Template'))
    recipient = EmailLookup('emails', label=_('Recipient list'))
    extra_message = forms.CharField(
        widget=Textarea, required=False, label=_('Extra message'))

    uri = forms.CharField(widget=forms.HiddenInput)
    name = forms.CharField(widget=forms.HiddenInput)
    pk = forms.CharField(widget=forms.HiddenInput)



from django.contrib import admin
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
import json

from async_notifications.tasks import send_email
from django.conf import settings
from .forms import NotificationForm, TemplateForm
from .models import EmailNotification, EmailTemplate, TemplateContext
from .settings import TEXT_AREA_WIDGET
from .utils import extract_emails


class UserAdminListFilter(admin.SimpleListFilter):
    title = _('User')
    parameter_name = 'userf'

    def lookups(self, request, model_admin):
        return (
            ('0', _('All mails')),
            ('1', _('My mails')),
        )

    def queryset(self, request, queryset):
        if self.value() == '1':
            return queryset.filter(user=request.user)
        return queryset

# Register your models here.
#from ckeditor.widgets import CKEditorWidget


class MyNotification(admin.ModelAdmin):

    fields = (("enqueued", "sended", "problems"),
              "subject",
              "recipient", "bcc", 'cc',
              "message",
              "file"
              )

    list_display = ("subject", "recipient_emails", "enqueued",
                    "sended", "problems", 'create_datetime')

    readonly_fields = ['recipient_emails']
    actions = ['send_now']
    ordering = ['-create_datetime', 'sended', '-enqueued']

    date_hierarchy = 'create_datetime'

    form = NotificationForm

    def get_queryset(self, request):
        query = super(MyNotification, self).get_queryset(request)
        if not request.user.is_superuser:
            query = query.filter(user=request.user)
        return query

    def get_list_filter(self, request):
        filter = super(MyNotification, self).get_list_filter(request)
        if request.user.is_superuser:
            return [UserAdminListFilter]
        return filter

    def save_model(self, request, obj, form, change):
        obj.user = request.user
        obj.save()

    def send_now(self, request, queryset):
        for email in queryset:
            send_email.delay(email.pk)
    send_now.short_description = _("Send email now")

    def recipient_emails(self, obj):
        mails = extract_emails(obj.recipient)
        if len(mails) > 5:
            return ", ".join(mails[:5]) + str(_(" and more"))
        return ", ".join(mails)
    recipient_emails.short_description = _("Recipients")


class EmailTemplateAdmin(admin.ModelAdmin):
    field = ('code', 'subject', 'message', 'template_context')
    list_display = ('code', 'subject')
    search_fields = ['code', 'subject']
    readonly_fields = ['template_context']
    form = TemplateForm

    def template_context(self, obj=None):
        if obj is None:
            return ""
        try:
            tcontext = TemplateContext.objects.get(code=obj.code)
            context = json.loads(tcontext.context_dic)
        except TemplateContext.DoesNotExist as e:
            if settings.DEBUG:
                return obj.code+" -- "+str(e)
            return ""
        dev = ""
        for view_name in context:
             
            dev += "<h4>%s</h4><ol>" % (view_name)
            datacont = context[view_name]
            if isinstance(datacont, dict):
                datacont = datacont.items()
            for name, help_text in datacont:
                dev += "<li><strong>%s:</strong> %s</li><br>" % (name, help_text)
            dev += "</ol>"
 
        return mark_safe(dev)
    template_context.short_description = _("Template context")

# admin.site.register(TemplateContext)
admin.site.register(EmailTemplate, EmailTemplateAdmin)
admin.site.register(EmailNotification, MyNotification)

if settings.DEBUG:
    admin.site.register(TemplateContext)


# encoding: utf-8
from __future__ import unicode_literals

'''
Created on 21/12/2015

@author: luisza
'''
from django.db.models.signals import post_save
from async_notifications.models import EmailNotification
from django.dispatch.dispatcher import receiver
from async_notifications.tasks import send_email


@receiver(post_save, sender=EmailNotification)
def check_email(sender, **kwargs):
    obj = kwargs['instance']

    if kwargs['created'] and  obj.enqueued is False:
        send_email.delay(obj.pk)


# encoding: utf-8

'''
Free as freedom will be 25/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from ajax_select import register, LookupChannel
from django.db.models.query_utils import Q

from django.utils.translation import ugettext as _
from .utils import hexify, extract_emails, get_model
from .settings import (NOTIFICATION_GROUP_MODEL,
                       NOTIFICATION_USER_MODEL,
                       USER_LOOKUP_FIELDS,
                       GROUP_LOOKUP_FIELDS)

Group = get_model(NOTIFICATION_GROUP_MODEL)
User = get_model(NOTIFICATION_USER_MODEL)


class Person(object):
    email = None
    name = None

    def __init__(self, name, email):
        self.name = name
        self.email = email
        self.pk = hexify(email)

    def __str__(self):
        return "%s (%s)" % (self.name, self.email)


def get_filters(filters, q):
    fields = None
    for f in filters['filter']:
        if fields is None:
            fields = Q(**{f: q})
        else:
            fields |= Q(**{f: q})
    return fields


def get_display(obj, name):
    if "__" in name:
        names = name.split("__")
        objs = getattr(obj, names[0])
        return get_display(objs, "__".join(names[1:]))
    nname = getattr(obj, name)
    if callable(nname):
        nname = nname()
    return nname


@register('emails')
class NotificationEmailLookup(LookupChannel):
    model = User

    def get_query_groups(self, q, request):
        fields = get_filters(GROUP_LOOKUP_FIELDS, q)
        groups = Group.objects.filter(fields)
        gs = []
        for g in groups:
            name = get_display(g, GROUP_LOOKUP_FIELDS['display'])
            gs.append(Person(name, "%s@group" % name.replace(" ", "__")))
        return gs

    def get_query_user(self, q, request):

        fields = get_filters(USER_LOOKUP_FIELDS, q)
        users = self.model.objects.filter(
            fields
        ).order_by(USER_LOOKUP_FIELDS['order_by'])[:20]

        persons = [Person(get_display(u, USER_LOOKUP_FIELDS['display']),
                          u.email) for u in users if u.email]
        return persons

    def find_user(self, email):
        r = self.model.objects.filter(email=email)
        if len(r):
            user = r[0]
            return Person(get_display(user, USER_LOOKUP_FIELDS['display']),
                          user.email)
        return

    def find_group(self, email):
        name = email.replace("@group", "").replace("__", " ")
        fields = get_filters(GROUP_LOOKUP_FIELDS, name)
        g = Group.objects.filter(fields)
        if len(g):
            g = g[0]
            name = get_display(g, GROUP_LOOKUP_FIELDS['display'])
            return Person(name,
                          "%s@group" % name.replace(" ", "__")
                          )
        return

    def get_query(self, q, request):
        persons = self.get_query_user(q, request)
        persons += self.get_query_groups(q, request)

        if len(persons) == 0 and "@" in q:
            persons = [Person(_("Add external email"), q)]
        return persons

    def format_item_display(self, item):
        return "<span class='tag'>%s (%s)</span>" % (item.name, item.email)

    def get_objects(self, ids):
        objs = []

        if ids == []:
            return ids

        ids = "".join(ids)
        ids=ids.split(',')
        for email in extract_emails(ids):
            person = self.find_user(email)
            if person is None:
                person = self.find_group(email)
                if person is None:
                    person = Person(_("External user"), email)
            objs.append(person)
        return objs



# encoding: utf-8

'''
Free as freedom will be 25/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from django import forms
from .models import EmailNotification, EmailTemplate
from .settings import TEXT_AREA_WIDGET
from .widgets import EmailLookup
from django.utils.translation import ugettext_lazy as _
#from ajax_select.fields import AutoCompleteSelectMultipleField


class NotificationForm(forms.ModelForm):
    recipient = EmailLookup('emails', label=_('Recipient list'))
    bcc = EmailLookup('emails', label=_('Bcc Recipient list'))
    cc = EmailLookup('emails', label=_('CC Recipient list'))

    class Meta:
        model = EmailNotification
        fields = ("subject", "enqueued", "sended", "problems",
                  "message", "recipient", 'bcc', 'cc',
                  "file")
        widgets = {
            'message': TEXT_AREA_WIDGET
        }

class TemplateForm(forms.ModelForm):
    bcc = EmailLookup('emails', label=_('Bcc Recipient list'))
    cc = EmailLookup('emails', label=_('CC Recipient list'))

    class Meta:
        model = EmailTemplate
        fields = ('code', 'subject', 'message', 'bcc', 'cc')
        widgets = {
            'message': TEXT_AREA_WIDGET
        }


from django.shortcuts import render

# Create your views here.
from .forms import NotificationForm
from django.http.response import HttpResponse, Http404
from django.utils.translation import ugettext_lazy as _
from async_notifications.utils import send_email_from_template, extract_emails
from async_notifications.djcms_async_notifications.utils import get_str_base64url

from django.contrib.contenttypes.models import ContentType
from async_notifications.register import update_template_context
from async_notifications.models import EmailTemplate

update_template_context('page_email', 'Send page by email',
                        [('url', str(_('Article url'))),
                         ('page', str(
                             _('Article object [title, lead_in, \
                             content, author, owner, publishing_date, \
                             is_published, is_featured ]'))),
                         ('extra_message', str(_('User extra message'))),
                         ('user', str(_('Sender user')))]
                        )


def extract_context(form):
    uri = get_str_base64url(form.cleaned_data['uri'])

    app_label, model = form.cleaned_data['name'].split(".")
    page_type = ContentType.objects.get(app_label=app_label,
                                        model=model)
    page = page_type.get_object_for_this_type(pk=form.cleaned_data['pk'])

    return {'page': page, 'url': uri}


def get_email_modal(request):

    if request.method == 'POST':
        form = NotificationForm(request.POST)
        if form.is_valid():
            context = extract_context(form)
            context['extra_message'] = form.cleaned_data['extra_message']
            send_email_from_template(
                form.cleaned_data['template'].code,  # code
                extract_emails(form.cleaned_data['recipient']),
                enqueued=False,
                user=request.user,
                context=context
            )

            # #div_success is important because is how i comunicate with iframe
            return HttpResponse(
                _("""<div id="div_success">Email sent successfully</div>"""))
    else:
        uri = request.GET.get('uri', '')
        pk = request.GET.get('pk', '')
        name = request.GET.get('name', '')

        if not all([uri, pk, name]):
            raise Http404()
        initial = {'uri': uri, 'pk': pk, 'name': name}
        try:
            initial['template'] = EmailTemplate.objects.get(code='page_email')
        except:
            pass
        form = NotificationForm(initial=initial)

    response = render(request, 'djcms_async_notifications/page_form_email.html',
                      {
                          'form': form,
                      }
                      )
    # response.render()

    return response



# encoding: utf-8

'''
Free as freedom will be 26/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from .settings import (NOTIFICATION_USER_MODEL, USER_LOOKUP_FIELDS,
                       NOTIFICATION_GROUP_MODEL, GROUP_LOOKUP_FIELDS)
from .utils import extract_emails, get_model


#from django.contrib.auth.models import User, Group
User = get_model(NOTIFICATION_USER_MODEL)
Group = get_model(NOTIFICATION_GROUP_MODEL)


def get_mails_from_group(group_name):
    name = group_name.replace("@group", "").replace("__", " ")
    group = Group.objects.get(**{GROUP_LOOKUP_FIELDS['group_lookup']: name})
    email = None
    # check if group has email (util with mail list approach)
    name_field = GROUP_LOOKUP_FIELDS['email']

    if name_field:
        if hasattr(group, name_field):
            email = getattr(group, name_field)
    if email:
        return [email]

    users = User.objects.filter(**{USER_LOOKUP_FIELDS['group_lookup']: name})
    return [u.email for u in users]


def get_all_emails(text):
    if text is None:
        return []
    mails = extract_emails(text)
    gmails = []
    for mail in mails:
        if "@group" in mail:
            mails.remove(mail)
            gmails += get_mails_from_group(mail)
    mails += gmails
    return set(mails)



# encoding: utf-8
'''
Created on 20/12/2015

@author: luisza
'''

from __future__ import unicode_literals

from django.conf import settings
from django.core import mail
import importlib

from .mail_utils import get_all_emails
from .models import EmailNotification
from .settings import MAX_PER_MAIL


app = importlib.import_module(settings.CELERY_MODULE).app


def _send_email(obj, mails, bcc=None, cc=None):
    send_ok = False
    with mail.get_connection() as connection:
        message = mail.EmailMessage(obj.subject, obj.message,
                                    settings.DEFAULT_FROM_EMAIL,
                                    mails,
                                    bcc=bcc,
                                    cc=cc,
                                    connection=connection
                                    )
        message.content_subtype = "html"
        if obj.file:
            message.attach_file(settings.MEDIA_ROOT + obj.file.name)
        try:
            message.send()
            send_ok = True
        except:
            obj.problems = True
            obj.save()

    if send_ok:
        obj.sended = True
        obj.queued = False
        obj.problems = False
        obj.save()


@app.task
def send_email(obj):
    """
    Envía un correo 
    obj puede ser un pk o una instancia EmailNotification
    """
    if not isinstance(obj, EmailNotification):
        try:
            obj = EmailNotification.objects.get(pk=obj)
        except:
            return

    mails = list(get_all_emails(obj.recipient))
    bcc = list(get_all_emails(obj.bcc))
    cc = list(get_all_emails(obj.cc))
    while len(mails) > MAX_PER_MAIL:
        s_mails = mails[:MAX_PER_MAIL]
        mails = mails[MAX_PER_MAIL:]
        _send_email(obj, s_mails, bcc=bcc, cc=cc)
    if len(mails) > 0:
        _send_email(obj, mails, bcc=bcc, cc=cc)


@app.task
def send_daily():
    for email in EmailNotification.objects.filter(sended=False, enqueued=True):
        send_email(email)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-21 05:41
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=500, verbose_name='Subject')),
                ('message', models.TextField(verbose_name='Message')),
                ('recipient', models.TextField(verbose_name='Recipient list')),
                ('enqueued', models.BooleanField(default=True, verbose_name='Enqueued')),
                ('sended', models.BooleanField(default=False, verbose_name='Sended')),
                ('file', models.FileField(blank=True, null=True, upload_to='email/%Y/%M', verbose_name='File')),
                ('problems', models.BooleanField(default=False, verbose_name='Problems')),
                ('create_datetime', models.DateTimeField(auto_now_add=True, verbose_name='Create datetime')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=20, verbose_name='Code')),
                ('subject', models.CharField(max_length=500, verbose_name='Subject')),
                ('message', models.TextField(verbose_name='Message')),
            ],
        ),
    ]
